#---------------------------------------------------------------------------
#
#   libtunepimp -- The MusicBrainz tagging library.  
#                  Let a thousand taggers bloom!
#   
#   Copyright (C) Robert Kaye 2003
#   
#   This file is part of libtunepimp.
#
#   libtunepimp is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   libtunepimp is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with libtunepimp; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   $Id: metadata.py,v 1.6 2004/05/29 23:47:46 robert Exp $
#
#---------------------------------------------------------------------------

from ctypes import *

class metadataInternal(Structure):
    '''This class is used to get/set UTF-8 data from/to libtunepimp. Don't use this class directly!'''

    TP_ARTIST_NAME_LEN = 255
    TP_ALBUM_NAME_LEN = 255
    TP_TRACK_NAME_LEN = 255
    TP_ID_LEN = 40
    TP_FORMAT_LEN = 32
    TP_COUNTRY_LEN = 3
    TP_STATUS_LEN = 32
    TP_TYPE_LEN = 32

    artistNameType = c_char * TP_ARTIST_NAME_LEN
    albumNameType = c_char * TP_ALBUM_NAME_LEN
    trackNameType = c_char * TP_TRACK_NAME_LEN
    idType = c_char * TP_ID_LEN
    fileFormatType = c_char * TP_FORMAT_LEN
    releaseCountryType = c_char * TP_COUNTRY_LEN

    _fields_ = [
                ("artist", artistNameType),
                ("sortName", artistNameType),
                ("album", albumNameType),
                ("track", trackNameType),
                ("trackNum", c_int),
                ("variousArtist", c_int),
                ("artistId", idType),
                ("albumId", idType),
                ("trackId", idType),
                ("fileTrm", idType),
                ("albumArtistId", idType),
                ("duration", c_ulong),
                ("albumType", c_int),
                ("albumStatus", c_int),
                ("fileFormat", fileFormatType),
                ("releaseYear", c_int),
                ("releaseDay", c_int),
                ("releaseMonth", c_int),
                ("releaseCountry", releaseCountryType)
               ]
    def __init__(self, encoding):
        self.encoding = encoding

    def set(self, mdata):
        self.artist = mdata.artist.encode('utf-8')
        self.sortName = mdata.sortName.encode('utf-8')
        self.album = mdata.album.encode('utf-8')
        self.track = mdata.track.encode('utf-8')
        self.trackNum = mdata.trackNum
        self.variousArtist = mdata.variousArtist
        self.artistId = mdata.artistId.encode('utf-8')
        self.albumId = mdata.albumId.encode('utf-8')
        self.trackId = mdata.trackId.encode('utf-8')
        self.fileTrm = mdata.fileTrm.encode('utf-8')
        self.albumArtistId = mdata.albumArtistId.encode('utf-8')
        self.duration = mdata.duration
        self.albumType = mdata.albumType
        self.albumStatus = mdata.albumStatus
        self.fileFormat = mdata.fileFormat.encode('utf-8')
        self.releaseYear = mdata.releaseYear
        self.releaseDay = mdata.releaseDay
        self.releaseMonth = mdata.releaseMonth
        self.releaseCountry = mdata.releaseCountry.encode('utf-8')
        
    def get(self, mdata):
        mdata.artist = self.artist.decode(self.encoding)
        mdata.sortName = self.sortName.decode(self.encoding)
        mdata.album = self.album.decode(self.encoding)
        mdata.track = self.track.decode(self.encoding)
        mdata.trackNum = self.trackNum
        mdata.variousArtist = self.variousArtist
        mdata.artistId = self.artistId.decode(self.encoding)
        mdata.albumId = self.albumId.decode(self.encoding)
        mdata.trackId = self.trackId.decode(self.encoding)
        mdata.fileTrm = self.fileTrm.decode(self.encoding)
        mdata.albumArtistId = self.albumArtistId.decode(self.encoding)
        mdata.duration = self.duration
        mdata.albumType = self.albumType
        mdata.albumStatus = self.albumStatus
        mdata.fileFormat = self.fileFormat.decode(self.encoding)
        mdata.releaseYear = self.releaseYear
        mdata.releaseDay = self.releaseDay
        mdata.releaseMonth = self.releaseMonth
        mdata.releaseCountry = self.releaseCountry.decode(self.encoding)

class metadata(object):
    '''This class is used to get/set the metadata for a track inside tunepimp. For details
       on how to use this class, please look up the main tunepimp documentation'''


    def __init__(self, tunePimp):
        self.encoding = tunePimp.getEncoding()
        self.tplib = tunePimp.tplib
        self.tplib.md_ConvertToAlbumStatus.argtypes = [ c_char_p ]
        self.tplib.md_ConvertToAlbumType.argtypes = [ c_char_p ]

        self.artist = self.conv(u'')
        self.sortName = self.conv(u'')
        self.album = self.conv(u'')
        self.track = self.conv(u'')
        self.trackNum = 0
        self.variousArtist = 0
        self.artistId = self.conv(u'')
        self.albumId = self.conv(u'')
        self.trackId = self.conv(u'')
        self.fileTrm = self.conv(u'')
        self.albumArtistId = self.conv(u'')
        self.duration = 0
        self.albumType = 0
        self.albumStatus = 0
        self.fileFormat = self.conv(u'')
        self.releaseYear = 0
        self.releaseDay = 0
        self.releaseMonth = 0
        self.releaseCountry = self.conv(u'')

    def getEncoding(self):
        return self.encoding

    def conv(self, text):

        if text.__class__.__name__ == 'int':
            text = unicode(text)

        if text.__class__.__name__ == 'unicode' and self.encoding != 'unicode':
            return text.encode(self.encoding)
        else:
            return text

    def clear(self):
        internal = metadataInternal(self.encoding)
        internal.get(self)
        self.tplib.md_Clear(byref(internal))
        internal.set(self)

    def compare(self, other):
        internal = metadataInternal(self.encoding)
        internal.set(self)
        otherInternal = metadataInternal(self.encoding)
        otherInternal.set(other)
        return self.tplib.md_Compare(byref(internal), byref(otherInternal))

    def convertToAlbumStatus(self, statusStr):
        return self.tplib.md_ConvertToAlbumStatus(statusStr)

    def convertToAlbumType(self, typeStr):
        return self.tplib.md_ConvertToAlbumType(typeStr)

    def convertFromAlbumStatus(self, status):
        text = c_buffer(self.TP_STATUS_LEN)
        len = c_int(self.TP_STATUS_LEN)
        self.tplib.md_ConvertFromAlbumStatus(status, text, len)
        return text.value

    def convertFromAlbumType(self, type):
        text = c_buffer(self.TP_TYPE_LEN)
        len = c_int(self.TP_TYPE_LEN)
        self.tplib.md_ConvertFromAlbumType(type, text, len)
        return text.value
